From 12f1bd6306fbad3aeaa527310919b6041907e2f5 Mon Sep 17 00:00:00 2001
From: chenqiwu <chenqiwu@xiaomi.com>
Date: Mon, 4 Nov 2019 15:39:53 +0800
Subject: [PATCH] Fix a potential segfault for the ARM64 "bt -S
 <stack-address>" command

Fix for the ARM64 "bt -S <stack-address>" command to prevent
a potential segmentation violation due to an invalid exception
frame before transitioning to the process stack.

Signed-off-by: chenqiwu <chenqiwu@xiaomi.com>
---
 arm64.c | 20 ++++++++++++--------
 1 file changed, 12 insertions(+), 8 deletions(-)

diff --git a/arm64.c b/arm64.c
index 5ee5f1a..eaa2094 100644
--- a/arm64.c
+++ b/arm64.c
@@ -2590,10 +2590,12 @@ arm64_back_trace_cmd(struct bt_info *bt)
 			goto complete_user;
 
 		if (DUMPFILE() && is_task_active(bt->task)) {
-			exception_frame = stackframe.fp - KERN_EFRAME_OFFSET;
-			if (arm64_is_kernel_exception_frame(bt, exception_frame))
-				arm64_print_exception_frame(bt, exception_frame, 
-					KERNEL_MODE, ofp);
+			if (arm64_in_exception_text(stackframe.pc) && INSTACK(stackframe.fp, bt)) {
+				exception_frame = stackframe.fp - KERN_EFRAME_OFFSET;
+				if (arm64_is_kernel_exception_frame(bt, exception_frame))
+					arm64_print_exception_frame(bt, exception_frame,
+						KERNEL_MODE, ofp);
+			}
 		}
 	}
 
@@ -2710,10 +2712,12 @@ user_space:
 		}
 
 		if (DUMPFILE() && is_task_active(bt->task)) {
-			exception_frame = stackframe.fp - SIZE(pt_regs);
-			if (arm64_is_kernel_exception_frame(bt, exception_frame))
-				arm64_print_exception_frame(bt, exception_frame, 
-					KERNEL_MODE, ofp);
+			if (arm64_in_exception_text(stackframe.pc) && INSTACK(stackframe.fp, bt)) {
+				exception_frame = stackframe.fp - SIZE(pt_regs);
+				if (arm64_is_kernel_exception_frame(bt, exception_frame))
+					arm64_print_exception_frame(bt, exception_frame,
+						KERNEL_MODE, ofp);
+			}
 		}
 	}
 
-- 
1.9.1

